package com.circuitengine.view;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.JFrame;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;

import com.circuitengine.model.Components.Ammeter;
import com.circuitengine.model.Components.CapacitanceMeter;
import com.circuitengine.model.Components.CurrentIntegrator;
import com.circuitengine.model.Components.ElectricComponent;
import com.circuitengine.model.Components.InductanceMeter;
import com.circuitengine.model.Components.Ohmmeter;
import com.circuitengine.model.Components.VoltageIntegrator;
import com.circuitengine.model.Components.Voltmeter;

/*AUTHORS:

- Kevin Stueve (2009-12-20): initial published version
- Kevin Stueve (2019-08-30): converted from JApplet to JFrame
#*****************************************************************************
#       Copyright (C) 2009 Kevin Stueve kstueve@uw.edu
#
#  Distributed under the terms of the GNU General Public License (GPL)
#                  http://www.gnu.org/licenses/
#*****************************************************************************
*/
public final class CircuitEngineJFrame extends JFrame{
	public static void main(String[] args) {
		CircuitEngineJFrame jFrame = new CircuitEngineJFrame();
		jFrame.myInit();
		jFrame.setBounds(0, 0, 800, 700);
		jFrame.setVisible(true);
		jFrame.setDefaultCloseOperation(EXIT_ON_CLOSE);
	}
	
  public void myInit(){
    final CircuitEnginePanel circuitEnginePanel = new CircuitEnginePanel(this);
    this.getContentPane().add(circuitEnginePanel);
    JMenuBar menuBar = new JMenuBar();
    JMenu menu = new JMenu("Options and About");
    JMenuItem solverOptionsMenuItem = new JMenuItem("Numerical Solver Options");
    JCheckBoxMenuItem cbMenuItem = new JCheckBoxMenuItem("Mark Capacitors and Inductors with +");
    JCheckBoxMenuItem timeMenuItem = new JCheckBoxMenuItem("Time Passes");
    JMenuItem resetTimeMenuItem = new JMenuItem("Reset Time to 0s (Clears all Graphs)");
    JMenuItem resetCompsMenuItem = new JMenuItem("Reset all Meters, Inductors and Capacitors");
    JMenuItem aboutMenuItem= new JMenuItem("About");
    solverOptionsMenuItem.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent e){
        new ChangeErrorFrame(circuitEnginePanel,0,0);
      }
    });
    cbMenuItem.addItemListener(new ItemListener(){
      public void itemStateChanged(ItemEvent e){
        circuitEnginePanel.drawPlus=e.getStateChange()==ItemEvent.SELECTED;
        circuitEnginePanel.dp.repaint();
      }
    });
    timeMenuItem.addItemListener(new ItemListener(){
      public void itemStateChanged(ItemEvent e){
        circuitEnginePanel.timeOption=e.getStateChange()==ItemEvent.SELECTED;
        //circuitEnginePanel.dp.repaint();
      }
    });
    resetTimeMenuItem.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent e){
        synchronized(circuitEnginePanel.dp){
          circuitEnginePanel.dp.t=0;
          circuitEnginePanel.dp.repaint();
          for (PhasePlane phasePlane:circuitEnginePanel.phasePlanes){
            phasePlane.pb.clear(0);
          }
          for (ElectricComponent ec:circuitEnginePanel.myCircuit.electricComponents){
            if (ec.getClass()==CurrentIntegrator.class||ec.getClass()==Voltmeter.class
                ||ec.getClass()==Ammeter.class||ec.getClass()==VoltageIntegrator.class
                ||ec.getClass()==Ohmmeter.class||ec.getClass()==CapacitanceMeter.class
                ||ec.getClass()==InductanceMeter.class){
              if (ec.gf!=null&&ec.gf.pb!=null){
                ec.gf.pb.clear(0);
              }
            }
          }
        }
      }
    });
    resetCompsMenuItem.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent e){
        synchronized(circuitEnginePanel.dp){
          for (ElectricComponent ec:circuitEnginePanel.myCircuit.electricComponents){
            ec.secondaryValue=0;
            if (ec.getClass()==CurrentIntegrator.class){
              CurrentIntegrator ci= (CurrentIntegrator)ec;
              ci.chargeString="0.00 C";
              circuitEnginePanel.repaint();
            }
            if (ec.getClass()==VoltageIntegrator.class){
              VoltageIntegrator vi= (VoltageIntegrator)ec;
              vi.magneticFluxString="0.00 Wb";
              circuitEnginePanel.repaint();
            }
          }
        }
      }
    });
    aboutMenuItem.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent e){
        JOptionPane.showMessageDialog(null,
"Copyright \u00a9 2007-2008 Kevin Stueve kstueve@uw.edu.\n"+
"All rights reserved.\n"+
"IN NO EVENT SHALL KEVIN STUEVE BE LIABLE TO ANY PARTY\n"+
"FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES\n"+
"ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF\n"+
"KEVIN STUEVE HAS BEEN ADVISED OF THE POSSIBILITY OF\n"+
"SUCH DAMAGE.\n"+

"KEVIN STUEVE SPECIFICALLY DISCLAIMS ANY WARRANTIES,\n"+
"INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF\n"+
"MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE\n"+
"PROVIDED HEREUNDER IS ON AN \"AS IS\" BASIS, AND KEVIN\n"+
"STUEVE HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,\n"+
"ENHANCEMENTS, OR MODIFICATIONS.\n"+
"CircuitEngine incorporates the Ptplot 5.7 plotting software, which is released\n"+
"under the UC Berkeley copyright.\n"+

"Copyright (c) 1995-2008 The Regents of the University of California.\n"+
"All rights reserved.\n"+

"IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY\n"+
"FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES\n"+
"ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF\n"+
"THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF\n"+
"SUCH DAMAGE.\n"+

"THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,\n"+
"INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF\n"+
"MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE\n"+
"PROVIDED HEREUNDER IS ON AN \"AS IS\" BASIS, AND THE UNIVERSITY OF\n"+
"CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,\n"+
"ENHANCEMENTS, OR MODIFICATIONS."
,
                                      "About CircuitEngine",
                                      JOptionPane.PLAIN_MESSAGE);

      }
    });
    cbMenuItem.doClick();
    timeMenuItem.doClick();
    menu.add(solverOptionsMenuItem);
    menu.add(cbMenuItem);
    menu.add(timeMenuItem);
    menu.add(resetTimeMenuItem);
    menu.add(resetCompsMenuItem);
    menu.add(aboutMenuItem);
    
    menuBar.add(menu);
    this.setJMenuBar(menuBar);
  }
}
